''   Modified by Duane Degn
''   I added methods near the end of the program.
''   Uses object "tm1803" instead of the original jm_ws2812 object.
''   Swaps red and blue but not in a pretty way.
''   27 JAN 2015
''   
'' =================================================================================================
''
''   File....... jm_ws2812_seahawks.spin
''   Purpose.... WS2812/B demonstration program
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014-15 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 26 JAN 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us

  
                                                                  
con { io pins }                                                   
                                                                  
  RX1  = 31                                                      ' programming / terminal
  TX1  = 30                                                       
                                                                  
  SDA  = 29                                                      ' eeprom / i2c
  SCL  = 28                                                       
                                                                  
  LEDS = 15                                                      ' LED tx pin


con

  STRIP_LEN = 10'21
  BRIGHTNESS = $80              '' How bright should LEDs get?
                                '' The value of BRIGHTNESS
                                '' should be between zero and
                                '' 255 ($FF).          
  BLUE_ = $FF_00_00
  GREEN_ = $00_FF_00
  
obj

  strip : "tm1803"                                            ' WS2812 LED driver


var

  long green, blue

pub main | pos

  strip.start(LEDS, STRIP_LEN)                           ' start led driver
  strip.set_all(0)

  green := scale_rgb(GREEN_, BRIGHTNESS)
  blue := scale_rgb(BLUE_, BRIGHTNESS)
  
  init_duanes_stuff
  repeat    
    repeat 20
      color_wipe(green, 10)
      color_wipe(blue, 10)
    strip.set_all(0)
    pause(100)

    chaser(blue, green, 167, 20)     
    strip.set_all(0)
    pause(100)
    
    extra_from_duane(20_000) ' max allowed time is 26_500 longer intervals will cause rollover
    strip.set_all(0)
    pause(100)
                                              
          
pub chaser(color1, color2, ms, count) | ch

  repeat count
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' first
      strip.set(ch++, color2)
      strip.set(ch++, color1)
      strip.set(ch++, color1)
    pause(ms)
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' second
      strip.set(ch++, color1)
      strip.set(ch++, color2)
      strip.set(ch++, color1)
    pause(ms)
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' third
      strip.set(ch++, color1)
      strip.set(ch++, color1)
      strip.set(ch++, color2)
    pause(ms)
    
                                                        
con

  ' Routines ported from C code by Phil Burgess (www.paintyourdragon.com)


pub color_wipe(rgb, ms) | ch

'' Sequentially fills strip with color rgb
'' -- ms is delay between pixels, in milliseconds

  repeat ch from 0 to strip.num_pixels-1 
    strip.set(ch, rgb)
    pause(ms)


pub rainbow(ms) | pos, ch

  repeat pos from 0 to 255
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, wheel((pos + ch) & $FF))
    pause(ms)
    

pub rainbow_cycle(ms) | pos, ch 

  repeat pos from 0 to (255 * 5)
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, wheel(((ch * 256 / strip.num_pixels) + pos) & $FF))
    pause(ms)
    
pub wheel(pos)

'' Creates color from 0 to 255 position input
'' -- colors transition r->g->b back to r

  pos &= $FF

  if (pos < 85)
    return strip.color(255-pos*3, pos*3, 0)
  elseif (pos < 170)
    pos -= 85
    return strip.color(0, 255-pos*3, pos*3)
  else
    pos -= 170
    return strip.color(pos*3, 0, 255-pos*3)


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds
                                                                  
  if (ms < 1)                                                    ' delay must be > 0
    return                                                        
  else                                                            
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
 

pub scale_rgb(rgb, level)
'' From JonnyMac's "jm_ws2812

  if (level =< 0)
    return $00_00_00
    
  elseif (level < 255)
    rgb.byte[2] := rgb.byte[2] * level / 255
    rgb.byte[1] := rgb.byte[1] * level / 255 
    rgb.byte[0] := rgb.byte[0] * level / 255 
  
  return rgb
      
con '' constants used by Duane's additional code
  
  PSEUDO_MULTIPLIER = 1000 '' For pseudo real math, also referred to as scaled integer math.
  TOTAL_LEDS = 10
  HALF_RING = TOTAL_LEDS / 2
  MAX_HALF_INDEX = HALF_RING - 1
 
  MAX_LED_INDEX = TOTAL_LEDS - 1

  MIN_COLOR_INDEX = 0 
  MAX_COLOR_INDEX = 84 
  COLOR_RANGE = MAX_COLOR_INDEX - MIN_COLOR_INDEX

  MAX_RING_ROTATION_AMOUNT = TOTAL_LEDS / 2
  MAX_ROTATION_AMOUNT = TOTAL_LEDS / 2
  MAX_DIFFERENCE = COLOR_RANGE / 2
  MIN_DIFFERENCE = COLOR_RANGE / 6
  MAX_SPEED = 90 '100
  MIN_SPEED = 60 '80
                          
var '' global variables used by Duane's additional code

  long stripRotationInterval, stripSpeed, stripDelay
  long speedDirection, speedChangeInterval
  long speedChangeTimer, stripRotationTimer
  long finalColorIndex, initialColorIndex, difference
  long positionChange, differenceDirection 
  long ringAddress, position
  
pub init_duanes_stuff

  positionChange := 1
  differenceDirection := 1
  speedDirection := 1
  initialColorIndex := MIN_COLOR_INDEX
  finalColorIndex := MAX_COLOR_INDEX
  ringAddress := strip.address
  difference := MIN_DIFFERENCE
  stripSpeed := MIN_SPEED
  stripDelay := speedToDelay[stripSpeed]
  stripRotationInterval := MS_001 * stripDelay
  speedChangeInterval := clkfreq / 10

pub extra_from_duane(effectInterval) | initialColor, finalColor, localTime 

  effectInterval *= MS_001
  speedChangeTimer := cnt 
  stripRotationTimer := cnt
  localTime := cnt
        
  repeat 
 
    if CheckRingTime
      finalColorIndex := initialColorIndex + difference 
      if initialColorIndex > MAX_COLOR_INDEX '255 
        initialColorIndex -= COLOR_RANGE 'MAX_COLOR_INDEX 'MIN_COLOR_INDEX '255
        positionChange *= -1
      if finalColorIndex > MAX_COLOR_INDEX '255 
        finalColorIndex -= COLOR_RANGE 'MAX_COLOR_INDEX 'MIN_COLOR_INDEX '-= 255
      difference += differenceDirection
      if difference => MAX_DIFFERENCE
        differenceDirection := -1
      elseif difference =< MIN_DIFFERENCE
        differenceDirection := 1
      initialColor := WheelDim(initialColorIndex, BRIGHTNESS)
      finalColor := WheelDim(finalColorIndex, BRIGHTNESS)  
      Mirror(initialColor, finalColor, ringAddress, position)
      initialColorIndex++
      position += positionChange
      if  position > MAX_LED_INDEX
        position -= TOTAL_LEDS
      elseif position < 0
        position += TOTAL_LEDS
  while cnt - localTime < effectInterval
  
pub CheckRingTime
'' The "stripSpeed" variable will be changed by value of "speedDirection"
'' if an interval of time "speedChangeInterval" has transpired from
'' the last speed change.

  CheckStripSpeed ' check to see if the stripRotationInterval should be changed

  if cnt - stripRotationTimer < stripRotationInterval
    return 0
  else  
    stripRotationTimer += stripRotationInterval
    result := 1
    
pub CheckStripSpeed
'' The "stripSpeed" variable will be changed by value of "speedDirection"
'' if an interval of time "speedChangeInterval" has transpired from
'' the last speed change.

  if cnt - speedChangeTimer < speedChangeInterval
    return
  else        
    stripSpeed += speedDirection
    if stripSpeed => MAX_SPEED
      speedDirection := -1
    elseif stripSpeed =< MIN_SPEED
      speedDirection := 1     
    stripDelay := speedToDelay[stripSpeed]
    stripRotationInterval := MS_001 * stripDelay 
    speedChangeTimer += speedChangeInterval
    
pub Mirror(initialColor, finalColor, localPtr, mirrorPosition) | localIndex, scratchBuffer[HALF_RING]
'' Fill long buffer at "localPtr" with a mirrored colors.

  repeat while mirrorPosition > MAX_LED_INDEX
    mirrorPosition -= TOTAL_LEDS
  MergeColors(initialColor, finalColor, @scratchBuffer, HALF_RING)
 
  result := mirrorPosition - 1
  repeat localIndex from 0 to MAX_HALF_INDEX
    if result < 0
      result += TOTAL_LEDS
    if mirrorPosition > MAX_LED_INDEX
      mirrorPosition -= TOTAL_LEDS
             
    long[localPtr][mirrorPosition++] := scratchBuffer[localIndex]
    long[localPtr][result--] := scratchBuffer[localIndex]  
 
pub WheelDim(pos, localBrightness)

'' Creates color from 0 to 255 position input
'' -- colors transition r-g-b back to r
'' This version also adjusts the brightness of the LED.
'' Color resolution is lost when using the brightness
'' feature.

  if (pos < 85)
    return strip.colorx(255-pos*3, pos*3, 0, localBrightness)
  elseif (pos < 170)
    pos -= 85
    return strip.colorx(0, 255-pos*3, pos*3, localBrightness)
  else
    pos -= 170
    return strip.colorx(pos*3, 0, 255-pos*3, localBrightness)

pub MergeColors(initialColor, finalColor, distinationPtr, localSize) | tempColor[3], {
    } colorMultiplier, localIndex, initialColors[3], finalColors[3], pseudoIncrements[3]
'' localSize should include the initialColor but not finalColor.
  
  BustColors(initialColor, @initialColors)
  BustColors(finalColor, @finalColors)
   
  ' find incremental color change
  ' use the "PSEUDO_MULTIPLIER" mulitplier to reduce rounding errors
  ' Note we divide by multiplier prior to assigning color
  ' to expanded pattern buffer.
  repeat result from 0 to 2   
    pseudoIncrements[result] := ((finalColors[result] - initialColors[result])* PSEUDO_MULTIPLIER) / localSize 
  
  colorMultiplier := 1
   
  ' assign initial color
  long[distinationPtr] := initialColor
   
  ' assign between colors
  ' final color not copied here. It will be copied as the
  ' initial color of the next loop
  repeat localSize - 1
    distinationPtr += 4
    repeat result from 0 to 2
      tempColor[result] := initialColors[result] + ((pseudoIncrements[result] * {
       } colorMultiplier) / PSEUDO_MULTIPLIER)
    ' assign altered color
    long[distinationPtr] := Strip.Color(tempColor[0], tempColor[1], tempColor[2])
    colorMultiplier++


pub BustColors(localColor, localPtr)
'' Split the a color value of "localColor" into its components parts and store these values
'' in three sequential longs located at "localPtr".

  long[localPtr] := localColor.byte[2]
  long[localPtr + 4] := localColor.byte[1]
  long[localPtr + 8] := localColor.byte[0]
 
pub DimColor(localColor)


dat 'speed to delay lookup table, values are milliseconds
'' Lookup table from another project. This project would benefit from a different table.

speedToDelay            long 2000, 1951, 1904, 1857, 1811, 1765, 1721, 1677, 1635, 1593
                        long 1551, 1511, 1471, 1432, 1394, 1357, 1320, 1284, 1249, 1214
                        long 1180, 1147, 1115, 1083, 1051, 1021, 991, 961, 933, 904
                        long 877, 850, 824, 798, 772, 748, 723, 700, 677, 654
                        long 632, 610, 589, 569, 549, 529, 510, 491, 473, 455
                        long 438, 421, 404, 388, 373, 358, 343, 328, 314, 301
                        long 287, 275, 262, 250, 238, 227, 216, 205, 194, 184
                        long 175, 165, 156, 147, 139, 131, 123, 115, 108, 101
                        long 94, 88, 82, 76, 70, 65, 59, 55, 50, 46
                        long 41, 37, 34, 30, 27, 24, 21, 18, 16, 14
extraJustIncase         long 12, 10, 8, 6, 4, 3, 2, 1, 0

dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}